function getType(x) {
    return Object.prototype.toString.call(x).replace('object ', '');
}
const _actionTypePrefix = '@prim';
function stringify(x) {
    const type = getType(x);
    if (['[Number]', '[Boolean]', '[Undefined]'].indexOf(type) > 0) {
        return x;
    }
    return type;
}
function querify(payload = {}) {
    const payloadStr = stringify(payload);
    if (payloadStr === '[Object]') {
        return Object.keys(payload)
            .map(key => `${key}=${stringify(payload[key])}`)
            .join('&');
    }
    return payloadStr;
}
function updaterActionCreators(namespace) {
    function getMeta(updaterName) {
        return {
            isPrimAction: true,
            namespace,
            updaterName
        };
    }
    const ns = namespace;
    function createAction(updaterName, payload) {
        return {
            type: `${_actionTypePrefix}/${ns}/${updaterName}/?${querify(payload)}`,
            payload,
            meta: getMeta(updaterName)
        };
    }
    return {
        initState: (state) => {
            return createAction('initState', state);
        },
        setState: (payload) => {
            return createAction('setState', payload);
        },
        mergeState: (payload) => {
            return createAction('mergeState', payload);
        }
    };
}
function isMatchedAction(namespace, action) {
    const meta = action === null || action === void 0 ? void 0 : action.meta;
    return (meta === null || meta === void 0 ? void 0 : meta.isPrimAction) && (meta === null || meta === void 0 ? void 0 : meta.namespace) === namespace;
}
export default function createSlice(namespace, getDefaultState, creator) {
    if (typeof creator !== 'function') {
        throw new Error('Expected the creator to be a function.');
    }
    const actions = creator(updaterActionCreators(namespace));
    const reducer = {
        [namespace]: (state = getDefaultState(), action) => {
            if (!isMatchedAction(namespace, action))
                return state;
            const { updaterName } = action === null || action === void 0 ? void 0 : action.meta;
            const _updaters = {
                initState({ action, getDefaultState }) {
                    return Object.assign({}, getDefaultState(), action.payload);
                },
                setState({ state, action }) {
                    if (typeof action.payload === 'function') {
                        let reducer = action.payload;
                        return reducer(state);
                    }
                    return Object.assign({}, state, action.payload);
                },
                mergeState({ state, action }) {
                    const payload = action.payload;
                    return Object.keys(payload).reduce(function (s, key) {
                        if (getType(s[key]) === '[Object]' && getType(payload[key]) === '[Object]') {
                            s[key] = Object.assign({}, s[key], payload[key]);
                        }
                        else {
                            s[key] = payload[key];
                        }
                        return s;
                    }, Object.assign({}, state));
                }
            };
            if (updaterName) {
                return _updaters[updaterName]({
                    state: state,
                    action: action,
                    getDefaultState
                });
            }
            if (!reducer) {
                throw new Error(`reducer function is not defined in createPrimReducer("${namespace}", getDefaultState, reducer)`);
            }
        }
    };
    return { actions, reducer, selector: (state) => state[namespace] };
}
//# sourceMappingURL=redux-prim.js.map